import pytest
from mysqld_integration_test import Mysqld
from mysqld_integration_test.helpers import Utils


@pytest.fixture
def mysqld_connect(autouse=True):
    return Mysqld()


@pytest.fixture
def version_mariadb():
    return "mysqld  Ver 10.5.16-MariaDB for Linux on x86_64 (MariaDB Server)"


@pytest.fixture
def version_mysql():
    return "/usr/sbin/mysqld  Ver 8.0.32-0ubuntu0.20.04.2 for Linux on x86_64 ((Ubuntu))"


@pytest.mark.helper_test
def test_utils_class():
   utils = Utils()
   assert True


@pytest.mark.helper_test
def test_find_program():
    mysqld_location = Utils.find_program("mysqld")
    assert mysqld_location is not None


@pytest.mark.helper_test
def test_unused_port_isnum():
    port = Utils.get_unused_port()
    assert isinstance(port, int)


@pytest.mark.helper_test
def test_unused_port_isinrange():
    port = Utils.get_unused_port()
    assert ((port > 1024) and (port < 65535))


# Test for MariaDB variant
@pytest.mark.helper_test
def test_parse_version_mariadb_variant(version_mariadb):
    (variant, version_major, version_minor) = Utils.parse_version(version_mariadb)
    assert variant == "mariadb"


# Test for MariaD Bversion major number, also verifies it is an integer
@pytest.mark.helper_test
def test_parse_version_mariadb_major(version_mariadb):
    (variant, version_major, version_minor) = Utils.parse_version(version_mariadb)
    assert version_major == 10


# Test for MariaDB minor version
@pytest.mark.helper_test
def test_parse_version_mariadb_minor(version_mariadb):
    (variant, version_major, version_minor) = Utils.parse_version(version_mariadb)
    assert version_minor == '5.16'


# Test for MySQL variant
@pytest.mark.helper_test
def test_parse_version_mysql_variant(version_mysql):
    (variant, version_major, version_minor) = Utils.parse_version(version_mysql)
    assert variant == "mysql"


# Test for MySQL version major number, also verifies it is an integer
@pytest.mark.helper_test
def test_parse_version_mysql_major(version_mysql):
    (variant, version_major, version_minor) = Utils.parse_version(version_mysql)
    assert version_major == 8


# Test for MySQL minor version
@pytest.mark.helper_test
def test_parse_version_mysql_minor(version_mysql):
    (variant, version_major, version_minor) = Utils.parse_version(version_mysql)
    assert version_minor == '0.32'



import pytest
import functools
from mysqld_integration_test import Mysqld


@pytest.fixture
def mysqld_connect(autouse=True):
    return Mysqld()


def rgetattr(obj, attr, *args):
    def _getattr(obj, attr):
        return getattr(obj, attr, *args)
    return functools.reduce(_getattr, [obj] + attr.split('.'))


def rsetattr(obj, attr, val):
    pre, _, post = attr.rpartition('.')
    return setattr(rgetattr(obj, pre) if pre else obj, post, val)


# Make sure config options exists and check some defaults
@pytest.mark.settings_test
def test_settings_test(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config') is not None


@pytest.mark.settings_test
def test_dirs_basedir_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.dirs.base_dir') is not None


@pytest.mark.settings_test
def test_dirs_datadir_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.dirs.data_dir') is not None


@pytest.mark.settings_test
def test_dirs_etcdir_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.dirs.etc_dir') is not None


@pytest.mark.settings_test
def test_dirs_tmpdir_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.dirs.tmp_dir') is not None


@pytest.mark.settings_test
def test_database_host_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.host') is not None


@pytest.mark.settings_test
def test_database_port_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.port') is not None


@pytest.mark.settings_test
def test_datbase_username_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.username') is not None


@pytest.mark.settings_test
def test_database_password_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.password') is not None


@pytest.mark.settings_test
def test_database_socketfile_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.socket_file') is not None


@pytest.mark.settings_test
def test_database_pidfile_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.pid_file') is not None


@pytest.mark.settings_test
def test_database_mysqldbinary_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.mysqld_binary') is not None


@pytest.mark.settings_test
def test_database_mysqlinstalldbbinary_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.database.mysql_install_db_binary') is not None


@pytest.mark.settings_test
def test_general_timeoutstart_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.general.timeout_start') is not None


@pytest.mark.settings_test
def test_general_timeoutstop_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.general.timeout_stop') is not None


@pytest.mark.settings_test
def test_general_loglevel_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.general.log_level') is not None


@pytest.mark.settings_test
def test_general_configfile_exists(mysqld_connect):
    assert rgetattr(mysqld_connect, 'config.general.config_file') is not None


# Test that a config option that we know doesn't exist does not exist
@pytest.mark.settings_test
def test_general_faketest_notexists(mysqld_connect):
    with pytest.raises(AttributeError):
        _ = rgetattr(mysqld_connect, 'config.general.faketest') is None


# Test a config option passed in as an argument works
@pytest.mark.settings_test
def test_arg_config_option_exists():
    mysqld = Mysqld(port='8888')
    assert mysqld.config.database.port == '8888'


# Test that a config file option works
@pytest.mark.settings_test
def test_config_file_option_exists():
    mysqld = Mysqld(config_file='tests/data/config_port.cfg')
    assert mysqld.config.database.port == '9999'



import os
import functools
import yaml
import subprocess

from mysqld_integration_test.helpers import Utils

config_settings = {}
config_settings['database'] = ['username', 'password', 'host', 'port', 'mysql_install_db_binary', 'mysqld_binary']
config_settings['general'] = ['timeout_start', 'timeout_stop', 'log_level', 'config_file']


def rgetattr(obj, attr, *args):
    def _getattr(obj, attr):
        return getattr(obj, attr, *args)
    return functools.reduce(_getattr, [obj] + attr.split('.'))


def rsetattr(obj, attr, val):
    pre, _, post = attr.rpartition('.')
    return setattr(rgetattr(obj, pre) if pre else obj, post, val)


def merge_configs(config, section, section_cfg):
    for arg in config_settings[section]:
        if arg in section_cfg:
            rsetattr(config, f"{section}.{arg}", section_cfg[arg])

    return config


def load_config_file(config_file):
    if os.path.exists(config_file):
        with open(config_file, "r", encoding='utf-8') as ymlfile:
            cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
        return cfg
    return {}


def parse_config(config, config_args):
    # See if there is a config file
    cfg = load_config_file(config.general.config_file)

    # Merge config together with args
    for section in config_settings:
        if section in cfg:
            config = merge_configs(config, section, cfg[section])

    # Merge in any class arguments
    for section in config_settings:
        config = merge_configs(config, section, config_args)

    return config


class ConfigAttribute():
    pass


class ConfigFile():
    def __init__(self, base_dir):
        self.dirs = ConfigAttribute()
        self.dirs.base_dir = base_dir
        self.dirs.data_dir = os.path.join(self.dirs.base_dir, 'var')
        self.dirs.etc_dir = os.path.join(self.dirs.base_dir, 'etc')
        self.dirs.tmp_dir = os.path.join(self.dirs.base_dir, 'tmp')

        self.database = ConfigAttribute()
        self.database.host = "127.0.0.1"
        self.database.port = Utils.get_unused_port()
        self.database.username = 'root'
        self.database.password = 'root'
        self.database.socket_file = os.path.join(self.dirs.tmp_dir, 'mysql.sock')
        self.database.pid_file = os.path.join(self.dirs.tmp_dir, 'mysqld.pid')
        self.database.mysqld_binary = Utils.find_program('mysqld')
        self.database.mysql_install_db_binary = Utils.find_program('mysql_install_db')

        # Get the mysql variant and version
        (variant, version_major, version_minor) = Utils.parse_version(
                subprocess.check_output([self.database.mysqld_binary, '--version']).decode("utf-8"))
        self.version = ConfigAttribute()
        self.version.variant = variant
        self.version.major = version_major
        self.version.minor = version_minor

        self.general = ConfigAttribute()
        self.general.timeout_start = 30
        self.general.timeout_stop = 30
        self.general.log_level = "INFO"
        self.general.config_file = 'mysqld-integration-test.cfg'


class ConfigInstance():
    def __init__(self, args):
        self.host = args['host']
        self.port = args['port']
        self.username = args['username']
        self.password = args['password']
        self.socket_file = args['socket_file']



import os
import re
import socket

BASEDIRS = ['/', '/usr', '/usr/local', '/opt/homebrew']


class Utils():
    @staticmethod
    def find_program(name):
        for basedir in ['usr', 'usr/local']:
            for subdir in ['bin', 'libexec', 'sbin', 'scripts']:
                path = os.path.join("/", basedir, subdir, name)
                if os.path.exists(path):
                    return path
        return None

    @staticmethod
    def get_unused_port():
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.bind(('localhost', 0))
        _, port = sock.getsockname()
        sock.close()

        return port

    @staticmethod
    def parse_version(version_str):
        version_info = (re.findall(r"Ver (\d+)\.([0-9.]+)\-(\w+)", version_str))
        version_major = int(version_info[0][0])
        version_minor = version_info[0][1]
        version_variant = version_info[0][2].lower()
        if version_major == 8:
            version_variant = "mysql"

        return (version_variant, version_major, version_minor)
